#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""\
Convert SVG that includes TexText objects to PDF + LaTeX.

Author: Håkon Marthinsen
"""

# Copyright (C) 2012 Håkon Marthinsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__version__ = '0.1.0'
__author__ = 'Håkon Marthinsen'

import inkex
import os
import sys
from textext import TexText, exec_command
from tempfile import mkstemp

import pygtk
pygtk.require('2.0')

import gtk

# Check for new pygtk: this is new class in PyGtk 2.4
if gtk.pygtk_version < (2, 3, 90):
    inkex.debug("PyGtk 2.3.90 or later required")
    raise SystemExit

TEXTEXT_NS = u"http://www.iki.fi/pav/software/textext/"
SVG_NS = u"http://www.w3.org/2000/svg"
SODIPODI_NS = u"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"


class TexTextExporter(inkex.Effect):
    def __init__(self):
        inkex.Effect.__init__(self)

        # Choose an output folder.
        dialog = gtk.FileChooserDialog(
            title='Save as a PDF document',
            action=gtk.FILE_CHOOSER_ACTION_SAVE,
            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                     gtk.STOCK_OPEN, gtk.RESPONSE_OK))

        dialog.set_default_response(gtk.RESPONSE_OK)

        filefilter = gtk.FileFilter()
        filefilter.set_name("PDF document")
        filefilter.add_mime_type("application/pdf")
        filefilter.add_pattern("*.pdf")
        dialog.add_filter(filefilter)

        filefilter = gtk.FileFilter()
        filefilter.set_name("All files")
        filefilter.add_pattern("*")
        dialog.add_filter(filefilter)

        response = dialog.run()

        if response == gtk.RESPONSE_OK:
            self.outputpdf = dialog.get_filename()
        elif response == gtk.RESPONSE_CANCEL:
            raise SystemExit
        dialog.destroy()

    def effect(self):
        svg_file = sys.argv[-1]
        stream = open(svg_file, 'r')
        document = inkex.etree.parse(stream)
        stream.close()

        nodes = document.getroot()

        # Create list of TexText nodes.
        textextnodes = []
        for node in nodes.iter('{%s}g' % SVG_NS):
            if '{%s}text' % TEXTEXT_NS in node.attrib:
                textextnodes.append(node)

        for node in textextnodes:
            # Add invisible dot at basepoint.
            code = node.get('{%s}text' % TEXTEXT_NS).decode('string-escape')
            code = '\\rlap{\\rule{.00001pt}{.00001pt}}' + code
            node.set('{%s}text' % TEXTEXT_NS, code.encode('string-escape'))

            # Regenerate the TexText object.
            e = TexText()
            e.getoptions()
            e.document = document
            e.getposinlayer()
            e.options.ids = [node.get('id')]
            e.selected = {e.options.ids[0]: node}
            e.options.text = code
            e.options.preamble_file = node.get('{%s}preamble' % TEXTEXT_NS)
            e.options.scale_factor = 1.25
            e.effect()

        # Since the document has changed, we must recreate list of TexText
        # nodes.
        textextnodes = []
        for node in nodes.iter('g'):
            if '{%s}text' % TEXTEXT_NS in node.attrib:
                textextnodes.append(node)

        for node in textextnodes:
            # Find the basepoint dot.
            for subnode in node.iter('path'):
                d = subnode.attrib['d'].split()
                if len(d) == 6:
                    if max(map(abs, map(float, d[1:3] + d[4:]))) < .01:
                        # Extract the numbers in the transform matrix.
                        transform \
                            = subnode.attrib['transform'][7:-1].split(',')
                        x, y = transform[-2:]

                        # Add a text element inside the TexText group node
                        # containing the original LaTeX code.
                        text = inkex.etree.SubElement(node, \
                                                      inkex.addNS('text', \
                                                                  'svg'), \
                                                      attrib={'x': x, 'y': y})
                        text.text = node.attrib['{%s}text' \
                            % TEXTEXT_NS].decode('string-escape')[32:]

                        # Copy the TexText group node style attribute to the
                        # text node.
                        style = node.get('style')
                        if style is not None:
                            text.set('style', style)

                        try:
                            del node.attrib['style']
                        except:
                            pass

                        # Delete the group containing the paths.
                        for el in node.iterchildren('g'):
                            node.remove(el)
                        # Delete the defs element.
                        for el in node.iterchildren('defs'):
                            node.remove(el)

        # Write document to temporary file.
        cwd = os.getcwd()

        outputfolder = os.path.dirname(self.outputpdf)

        os.chdir(outputfolder)

        tmp_file = mkstemp(dir=outputfolder, suffix='.svg')
        document.write(tmp_file[1])

        # Convert to PDF + LaTeX.
        exec_command(['inkscape', '-z',  '-A',
                      self.outputpdf, '--export-latex',
                      tmp_file[1]])

        os.remove(tmp_file[1])
        os.chdir(cwd)


if __name__ == '__main__':
    effect = TexTextExporter()
    effect.effect()
